/**
* @file Uzetherm.c
*
* @brief UzeTherm for Uzebox - Speaking thermometer with LM75
*
* @version 1.1
*
* @date 2012 
*
* @author  Hartmut Wendt  info@hwhardsoft.de.vu	<http://www.hwhardsoft.de.vu>
*
* @copyright GNU Public License V3
* 
* @note
*   This program is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   You should have received a copy of the GNU General Public License
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <stdbool.h>
#include <string.h>
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include "kernel/uzebox.h"
#include "kernel/mmc.h"
#include "i2c_software_master.h"
#include "uzewav.h"
#include "data/UzeThermTiles.pic.inc"
#include "data/fonts.pic.inc"
#include "data/UzeTherm.maps.inc"
#include "data/sprites.pic.inc"
#include "data/sprites.map.inc"



/*** global definitons ****************************************************************/


// program modes
/** 
 @brief program mode: shows the digital thermometer...
*/
#define PM_Thermometer 	1	
/** 
 @brief program mode: demonstration without sensor...
*/
#define PM_Demo   		2	


// temperature definitions
/** 
 @brief unit Celsius
*/
#define Celsius		0

/** 
 @brief unit Fahrenheit
*/
#define Fahrenheit	1

/** 
 @brief maximal temperature in Celsius
*/
#define Max_temp	50	// 50 C

/** 
 @brief minimal temperature in Celsius
*/
#define Min_temp	-20	// -20 C

/** 
 @brief start position in Y direction (= -20 degree Celsius) for thermometer bar
*/
#define Bar_Start_Y	21	

/** 
 @brief start position in X direction for thermometer bar
*/
#define Bar_Start_X	5	


// button
/** 
 @brief position in X direction for language button
*/
#define BUTTON_FLAG_X	17

/** 
 @brief position in Y direction for language button
*/
#define BUTTON_FLAG_Y	17

/** 
 @brief position in X direction for mute button
*/
#define BUTTON_SPK_X	23

/** 
 @brief position in Y direction for mute button
*/
#define BUTTON_SPK_Y	17


// language
/** 
 @brief definition for german voice output
*/
#define GERMAN	0

/** 
 @brief definition for english voice output
*/
#define ENGLISH	1


/** 
 @brief 8-bit, 255 period LFSR (for generating pseudo-random numbers)
*/
#define PRNG_NEXT() (prng = ((u8)((prng>>1) | ((prng^(prng>>2)^(prng>>3)^(prng>>4))<<7))))




/*** global variables *****************************************************************/
/** 
 @brief Pseudo-random number generator
*/
u8 prng; 

/** 
 @brief program mode 
*/
u8 program_mode=PM_Thermometer;	

/** 
 @brief choosed language 
*/
u8 language = GERMAN;

/** 
 @brief status for voice output / mute button
*/
bool SPK_ON = true;



// SD/mmc card
extern bool cardDetected;
extern File files[MAX_WAVES];
extern unsigned long sectorNo;
extern bool playing;
extern long bootRecordSector;
extern int reservedSectors;
extern int sectorsPerFat;
extern unsigned char maxRootDirectorySectors;
extern long dirTableSector;
extern char wav_queue[7];



/*** function prototypes *****************************************************************/
void init(void);
void set_PM_mode(u8 mode);
void copy_buf(unsigned char *BUFA, unsigned char *BUFB, unsigned char ucANZ);
void fill_buf(char *BUFA, char content, unsigned char ucANZ);
int gettemperature(void);
void convert_temperature(double *dTempOut, int iTempIn, u8 TempFormat);
void draw_thermometer_bar(int iTempRaw);
void draw_Dotmatrix(u8 PosX,u8 PosY, int iTempRaw, u8 TempFormat);
void VsyncCallBack(void);
void convert_temperature_DE(int iTempRaw, u8 TempFormat);
void convert_temperature_EN(int iTempRaw, u8 TempFormat);
void draw_button(u8 PosX, u8 PosY, const char *map, bool Pressed);
void button_handler(u8 PosX, u8 PosY);
static inline u8 detect_emulator();

/**
Call process mixer for sound output
@param void
@return non
*/
void VsyncCallBack(void){
	mmc_processMixer();
}

/**
init the kernel, graphic, SD card and I2C interface 
@param void
@return non
*/
void init(void)
{
  unsigned char tmp=0, progmode;
  
  // init tile table
  SetTileTable(BGTiles);
  // init font table
  SetFontTilesIndex(BGTILES_SIZE);
  // init Sprites
  SetSpritesTileTable(sprites_tileset);	


  ClearVram();
  Print(5,1,PSTR("UZETHERM VERSION 1.1"));
  Print(4,3,PSTR("(C) HARTMUT WENDT 2012"));

  Print(5,5,PSTR("WWW.HWHARDSOFT.DE.VU"));
  WaitVsync(60); 	

  //init random generator for demo mode
  prng = 70;

  // detect snes controllers	
  //DetectControllers() & 0x0C;	
  
  // detect UZEM emulator				
  if (detect_emulator()) {
    Print(1,8,PSTR("EMULATOR DETECTED!"));
    

  } else {

  // initialise SD card
    Print(1,8,PSTR("INITIALIZING SD..."));
    cardDetected=false;
    tmp = SD_init();
  
  	if (tmp == 0) {
		Print(19,8,PSTR("FAILED"));

  	} else if (tmp == 1) {
		Print(19,8,PSTR("FAILED"));
    	Print(6,9,PSTR("FIRST READ FAILED")); 

  	} else {
		Print(19,8,PSTR("OK"));
		cardDetected=true;
  	}
  }

  // searching files on SD card
  if (cardDetected) {
  	Print(1,11,PSTR("LOAD ENGLISH VOICES..."));

	fat_init(sector.buffer);
	//get directory table
	dirTableSector=bootRecordSector + reservedSectors + (sectorsPerFat * 2); 
    tmp=find_files(PSTR("ENG"));	
	if (tmp == 29) {
	  Print(23,11,PSTR("OK"));
      Print(1,14,PSTR("LOAD GERMAN VOICES..."));
      tmp=find_files(PSTR("GER"));	
	  if (tmp == 30) Print(22,14,PSTR("OK"));
	  else {
	    Print(22,14,PSTR("FAILED"));
	    cardDetected = false;			
	  }
	} else {
	  Print(23,11,PSTR("FAILED"));
	  cardDetected = false;	
    }
  }	

  if (cardDetected) {
     SetUserPreVsyncCallback(&VsyncCallBack); 
     SPK_ON = true;
  } else SPK_ON = false;

  // initialize I2C library	
  Print(1,17,PSTR("INITIALIZING SENSOR..."));
  // reset I2C pins
  PORTD &= ((1<<PIND3) | (1<<PIND5));
  WaitVsync(10);

  if ((PIND & (1<<PIND5)) && (PIND & (1<<PIND3))) {
    i2c_master_init();         
	progmode = PM_Thermometer;	
	Print(23,17,PSTR("OK"));  
  } else {	  
    Print(23,17,PSTR("FAILED"));  
	progmode = PM_Demo;	
		
  } 	

  WaitVsync(200); 
  set_PM_mode(progmode);	 
}



int main(){
int ibuttons=0,ibuttons_old;
u8 PosX=100, PosY=180;
char c1;
int iTempIN=20;
int ani_count=0;

  // init program
  init();        
  // proceed game	
  while(1)
  {
    WaitVsync(1);	  
    // get controller state
    ibuttons_old = ibuttons;
	ibuttons = ReadJoypad(0);

	// handle cursor
    if ((BTN_LEFT & ibuttons) && (PosX>0)) PosX -= 1;
    else if ((BTN_RIGHT & ibuttons) && (PosX<231)) PosX += 1;
    if ((BTN_UP & ibuttons) && (PosY>0)) PosY -= 1;
    else if ((BTN_DOWN & ibuttons) && (PosY<191)) PosY += 1;		
	MoveSprite(1,PosX,PosY,1,2);
	if (BTN_A & ibuttons) button_handler(PosX,PosY);

    switch(program_mode)
	{

	  // Thermometer
	  case PM_Thermometer:		
		if(playing) {
		   sectorNo=mmc_mixerGetCurrentSector();		    
		   stop_auto();
        } else {
		  if (!ani_count) {
	    	iTempIN = gettemperature();
			draw_thermometer_bar(iTempIN);
			draw_Dotmatrix(10,1, iTempIN, Celsius);
			draw_Dotmatrix(10,9, iTempIN, Fahrenheit);
		   	if (language == GERMAN) convert_temperature_DE(iTempIN, Celsius);
			else convert_temperature_EN(iTempIN, Celsius);
 			ani_count = 1200; // measure all 20s the temperature
		  }	
		  if (ani_count == 600) {
		   	if (language == GERMAN) convert_temperature_DE(iTempIN, Fahrenheit);
			else convert_temperature_EN(iTempIN, Fahrenheit);
          }
		  play_queue();
		}
		break;	  
	  
	  // Demo mode - no sensor 
	  case PM_Demo:
		if(playing) {
		   sectorNo=mmc_mixerGetCurrentSector();
		   stop_auto();
        } else {
		  if (!ani_count) {	
			// generate random number
			c1 = ((PRNG_NEXT() % 21) - 10);
			if ((c1 > 0) && ((iTempIN+c1) < 50)) iTempIN += c1;
			else if ((c1 < 0) && ((iTempIN-c1) > -20)) iTempIN += c1;
			
			draw_thermometer_bar(iTempIN * 256);
			draw_Dotmatrix(10,1, iTempIN * 256, Celsius);
			draw_Dotmatrix(10,9, iTempIN * 256, Fahrenheit);
		   	if (language == GERMAN) convert_temperature_DE(iTempIN*256, Celsius);
			else convert_temperature_EN(iTempIN*256, Celsius);
			ani_count = 1200; // measure all 20s the temperature
		  }
		  if (ani_count == 600) {
		   	if (language == GERMAN) convert_temperature_DE(iTempIN*256, Fahrenheit);
			else convert_temperature_EN(iTempIN*256, Fahrenheit);
          }
		  play_queue();		
		}
		break;

    }	

	if (ani_count) ani_count--;	// increase counter for animation
  }
  

} 


/**
set parameters, tiles, background etc for choosed program mode	
@param mode (program mode)
@return non
*/
void set_PM_mode(u8 mode) {

	switch (mode)
	{
	  
	  case	PM_Thermometer:
	  case 	PM_Demo:

 		ClearVram();
				
   		// shows digital Thermometer
		DrawMap2(1,1,Thermometer);
		draw_button(17,17, FLAG_DE, false); 
		if (SPK_ON) draw_button(23,17, SPK_enabled, false);
		else draw_button(23,17, SPK_disabled, false);
		
		MapSprite2(1,map_mouse,0);
		if (mode == PM_Demo) Print(11,25,PSTR("DEMO MODE"));


		break;
	}
	program_mode = mode;
}



/**
get background tile from vram
@param X coordinates of tile
@param Y coordinates of tile
@retval tile number
*/
u8 GetTile(u8 x, u8 y)
{

 return (vram[(y * 30) + x] - RAM_TILES_COUNT);

}


/**
read temperature from LM75 sensor via I2C bus
@param void
@retval temperature
*/
int gettemperature(void)
{
u8 LM75_H,LM75_L;
    i2c_master_start();			// Issue I2C signal: start
    i2c_master_write(0x90);		// Send Address byte
   	i2c_master_write(0x00);     // Send Pointer Reg. 0x00 = Temp
                                    //                   0x01 = Config.
                                    //                   0x02 = HYST Set Point
                                    //                   0x03 = OS Set Point


    i2c_master_start();;        // Issue a Re-start
    i2c_master_write(0x91);		// Issue a Read Command
    LM75_H = i2c_master_read(0);     // Read the high data byte, (1) = Mster ACK's
    LM75_L = i2c_master_read(1);     // Read the low data byte , (0) = Master No ACK's
    i2c_master_stop();          // Issue I2C signal: stop
	return((LM75_H * 256) + LM75_L);
}


/**
converts measured temperature into a value in Celsius or Fahrenheit
@param pointer to output temperature
@param measured (input) temperature from sensor
@param temperature unit (Fahrenheit or Celsius)
@return none
*/
void convert_temperature(double *dTempOut, int iTempIn, u8 TempFormat)
{

  if (iTempIn & 0x8000)
  {
    // negative temperature
    iTempIn &= 0x7FFF;    
    iTempIn >>= 7;
    iTempIn--;
    iTempIn ^= 0xFF; 
    iTempIn *= -1;
  } else {
    // positive temperature
    iTempIn >>= 7;
  }     
  // converts into double   	  
  if (TempFormat == Fahrenheit) *dTempOut = (iTempIn * 0.9) + 32;   // F
  else *dTempOut = iTempIn * 0.5;  // C     
  
} 


/**
draws the thermometer bar
@param measured (input) temperature from sensor
@return none
*/
void draw_thermometer_bar(int iTempRaw)
{
  
  u8 uc1, uc2; 
  double dTempIn;
  convert_temperature(&dTempIn,iTempRaw,Celsius);		

  if ((dTempIn > Max_temp) || (dTempIn < Min_temp)) return;

  // conversion in pixel	
  int iPixel = (dTempIn - Min_temp) * 2;

  // draw bar
  for(uc1 = 0; uc1 <= (iPixel / 8); uc1++) {
    DrawMap2(Bar_Start_X, Bar_Start_Y - uc1, Bar_Full); 
  }
  
  // draw bar tip	
  uc2 = iPixel % 8;
  DrawMap2(Bar_Start_X, Bar_Start_Y - uc1, Thermometer_bars[0] + (uc2 * 3)); 

  // erase upper part of bar
  Fill(Bar_Start_X,2,1,Bar_Start_Y - uc1 - 2,0);
  		  
}


/**
draws the dotmatrix style display
@param X coordinate for output (first digit, left corner)
@param Y coordinate for output (first digit, upper corner)
@param temperature unit (Fahrenheit or Celsius)
@return none
*/
void draw_Dotmatrix(u8 PosX,u8 PosY, int iTempRaw, u8 TempFormat)
{	
	u8 uc1;
    double dTempIn;
    convert_temperature(&dTempIn,iTempRaw,Celsius);
	int iTempIn = dTempIn;

	// check temperature range
	if ((iTempIn > Max_temp) || (iTempIn < Min_temp)) return;

	// check position
	if ((PosX > 10) || (PosY > 20)) return;

	convert_temperature(&dTempIn,iTempRaw,TempFormat);
	iTempIn = dTempIn;

	// draw the degree unity
	DrawMap2(PosX + 12, PosY, Dotmatrix_Degree);

	// draw the sign for temperature format
	if (TempFormat == Fahrenheit) DrawMap2(PosX + 16, PosY, Dotmatrix_F);
	else DrawMap2(PosX + 16, PosY, Dotmatrix_C);

	// draw the prefix/sign
	if (iTempIn < 0) {
		DrawMap2(PosX, PosY, Dotmatrix_Minus);
	    iTempIn *= -1;

    } else if (iTempIn > 99) {
		DrawMap2(PosX, PosY, Dotmatrix_1);

    } else DrawMap2(PosX, PosY, Dotmatrix_empty);


	// draw value medium digit
	if (iTempIn > 99) {
		iTempIn -= 100;
      	uc1 = iTempIn / 10;
		DrawMap2(PosX + 4, PosY, DotMatrix_Nums[0] + (uc1 * 30)); 

	} else {
   	  uc1 = iTempIn / 10;
	  if (uc1) DrawMap2(PosX + 4, PosY, DotMatrix_Nums[0] + (uc1 * 30)); 
	  else DrawMap2(PosX + 4, PosY, Dotmatrix_empty); 
    }

	// draw value last digit
	uc1 = iTempIn % 10;
    DrawMap2(PosX + 8, PosY, DotMatrix_Nums[0] + (uc1 * 30)); 

}


/**
draw a graphic button
@param X coordinate for button (first digit, left corner)
@param Y coordinate for button (first digit, upper corner)
@param map for button picture	
@param button is pressed or unpressed
@return none
*/
void draw_button(u8 PosX, u8 PosY, const char *map, bool Pressed) {
 	if (Pressed) DrawMap2(PosX, PosY, Frame_pressed);
	else DrawMap2(PosX, PosY, Frame_unpressed);
	DrawMap2(PosX + 1, PosY + 1, map);
}


/**
button handler for mouse cursor
@param X position of cursor
@param Y position of cursor
@return none
*/
void button_handler(u8 PosX, u8 PosY) {

  

	if (!cardDetected) return;


	// buttom language/ flag
	if ((PosX>= BUTTON_FLAG_X*8) && (PosX<= (BUTTON_FLAG_X+6)*8) &&
		(PosY>= BUTTON_FLAG_Y*8) && (PosY<= (BUTTON_FLAG_Y+6)*8)) {

		// stop actual voice output
		stopSong();

		// delete voice queue
		fill_buf(&wav_queue[0],-99,7);


		if (language == GERMAN) {
		  language = ENGLISH;
		  // press button
		  draw_button(BUTTON_FLAG_X, BUTTON_FLAG_Y, FLAG_GB,true); 	
		
		  // load english voice files	
		  find_files(PSTR("ENG"));
		  WaitVsync(20); 	
				
		  // unpress button
		  draw_button(BUTTON_FLAG_X, BUTTON_FLAG_Y, FLAG_GB,false); 	


		} else {
		  language = GERMAN;	
		  // press button
		  draw_button(BUTTON_FLAG_X, BUTTON_FLAG_Y, FLAG_DE,true); 	

		  // load german voice files	
		  find_files(PSTR("GER"));	
		  WaitVsync(20); 	
			
		  // unpress button
		  draw_button(BUTTON_FLAG_X, BUTTON_FLAG_Y, FLAG_DE,false); 	

		}


	// button speaker on/off
	} else if ((PosX>= BUTTON_SPK_X*8) && (PosX<= (BUTTON_SPK_X+6)*8) &&
			   (PosY>= BUTTON_SPK_Y*8) && (PosY<= (BUTTON_SPK_Y+6)*8)) {
	
		// stop actual voice output
		stopSong();

		// delete voice queue
		fill_buf(&wav_queue[0],-99,7);		

		// switch speaker off
		if (SPK_ON) {
		  SPK_ON = false;
		  	
		  // press button
		  draw_button(BUTTON_SPK_X, BUTTON_SPK_Y, SPK_disabled,true); 		
		  WaitVsync(25); 	
				
		  // unpress button
		  draw_button(BUTTON_SPK_X, BUTTON_SPK_Y, SPK_disabled,false); 	
		
		
		// switch speaker on
		} else {
		  SPK_ON = true;

		  // press button
		  draw_button(BUTTON_SPK_X, BUTTON_SPK_Y, SPK_enabled,true); 		
		  WaitVsync(25); 	
				
		  // unpress button
		  draw_button(BUTTON_SPK_X, BUTTON_SPK_Y, SPK_enabled,false); 	

		}			
	}
}



/**** V O I CE  O U T P U T ********************************************************/

/**
converts measured temperature into a german voice anouncement
@param measured (input) temperature from sensor
@param temperature unit (Fahrenheit or Celsius)
@return none
*/
void convert_temperature_DE(int iTempRaw, u8 TempFormat) 
{
	u8 uc1, pos=0;
    double dTempIn;
    convert_temperature(&dTempIn,iTempRaw,Celsius);
	int iTempIn = dTempIn;

	// voice output enabled & possible
	if ((!SPK_ON) || (!cardDetected)) return;

	// check temperature range
	if ((iTempIn > Max_temp) || (iTempIn < Min_temp)) return;

	convert_temperature(&dTempIn,iTempRaw,TempFormat);
	iTempIn = dTempIn;
	
	// clear voice queue
	fill_buf(&wav_queue[0],-99,7);

	// prefix/sign
	if (iTempIn < 0) {
		wav_queue[pos++] = ger_minus;
	    iTempIn *= -1;
	
	// or value > 99
    } else if (iTempIn > 99) {
		wav_queue[pos++] = ger_100;
		if (iTempIn == 100) goto Degree;
		iTempIn -= 100;
    } 


	// convert value 
	// 14
	if (iTempIn == 14) {
		wav_queue[pos++] = ger_14;

	// 16
	} else if (iTempIn == 16) {
		wav_queue[pos++] = ger_16;

	// 17
	} else if (iTempIn == 17) {
		wav_queue[pos++] = ger_17;
	
	// 13, 15, 17 - 19
	} else if ((iTempIn == 13) || (iTempIn == 15) || (iTempIn == 18) || (iTempIn == 19)) {
		wav_queue[pos++] = iTempIn % 10;
		wav_queue[pos++] = ger_10;

	// 20 - 99
	} else if ((iTempIn >= 20) && (iTempIn <= 99)) {
		uc1 = iTempIn % 10;
		if (uc1) {
		  wav_queue[pos++] = uc1;
		  wav_queue[pos++] = ger_und;
		}	
		wav_queue[pos++] = (iTempIn / 10) + ger_20 - 2;
		
	// 0 - 12
	} else if ((iTempIn >= 0) && (iTempIn <= 12)) {
		wav_queue[pos++] = iTempIn % 13;

    }

Degree:
	// degree 
	wav_queue[pos++] = ger_grad;

	// sign for temperature unity
	if (TempFormat == Fahrenheit) wav_queue[pos] = ger_fahrenheit;
	else wav_queue[pos] = ger_celsius;
}


/**
converts measured temperature into am english voice anouncement
@param measured (input) temperature from sensor
@param temperature unit (Fahrenheit or Celsius)
@return none
*/
void convert_temperature_EN(int iTempRaw, u8 TempFormat) 
{
	u8 uc1, pos=0;
    double dTempIn;
    convert_temperature(&dTempIn,iTempRaw,Celsius);
	int iTempIn = dTempIn;

	// voice output enabled & possible
	if ((!SPK_ON) || (!cardDetected)) return;

	// check temperature range
	if ((iTempIn > Max_temp) || (iTempIn < Min_temp)) return;

	convert_temperature(&dTempIn,iTempRaw,TempFormat);
	iTempIn = dTempIn;
	
	// clear voice queue
	fill_buf(&wav_queue[0],-99,7);

	// prefix/sign
	if (iTempIn < 0) {
		wav_queue[pos++] = eng_minus;
	    iTempIn *= -1;
	
	// or value > 99
    } else if (iTempIn > 99) {
		wav_queue[pos++] = eng_100;
		if (iTempIn == 100) goto Degree;
		iTempIn -= 100;
    } 

	// convert value 
	// 15
	if (iTempIn == 15) {
		wav_queue[pos++] = eng_15;
	
	// 14, 16 - 19
	} else if ((iTempIn == 14) || ((iTempIn >= 16) && (iTempIn <= 19))) {
		wav_queue[pos++] = iTempIn % 10;
		wav_queue[pos++] = eng_teen;

	// 20 - 99
	} else if ((iTempIn >= 20) && (iTempIn <= 99)) {
		wav_queue[pos++] = (iTempIn / 10) + eng_20 - 2;
		uc1 = iTempIn % 10;
		if (uc1) wav_queue[pos++] = uc1;	
		
	// 0 - 13
	} else if ((iTempIn >= 0) && (iTempIn <= 13)) {
		wav_queue[pos++] = iTempIn % 13;

    }

Degree:
	// degree 
	wav_queue[pos++] = eng_degree;

	// sign for temperature unity
	if (TempFormat == Fahrenheit) wav_queue[pos] = eng_fahrenheit;
	else wav_queue[pos] = eng_celsius;
}



/**** S T U F F ********************************************************************/

/**
copy a buffer into another buffer 
@param source buffer
@param target buffer
@param count of copied bytes
@return none
*/
void copy_buf(unsigned char *BUFA, unsigned char *BUFB, unsigned char ucANZ)
{
 for(;ucANZ>0 ; ucANZ--)
 {
  *(BUFB++) = *(BUFA++);
 }   
}


/**
fill a buffer 
@param target buffer
@param byte to fill
@param count of copied bytes
@return none
*/
void fill_buf(char *BUFA, char content, unsigned char ucANZ)
{
 for(;ucANZ>0 ; ucANZ--)
 {
  *(BUFA++) = content;
 }   
}


/**
detect if program run in UZEM 
@param void
@retval 0  program runs in real hardware
@retval 1  program runs in UZEM emulator
*/
static inline u8 detect_emulator()
{
   u8 x_out = 1;	

   asm volatile (
   	"ldi r26,0xaa"   "\n\t"
   	"sts 0xff,r26"   "\n\t"
   	"lds r25,0xff"   "\n\t"   	
   	"cpse r25,r26"   "\n\t"   
   	"clr %[out]"   "\n\t"
	: [out] "+r" (x_out)
	);
	return(x_out);
}

